from __future__ import annotations
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing import Self, Sequence, Literal
    from ._typing import _Area, _Areas

from aclib.cv import Image, FontLib, DotsetLib
from aclib.cv._typing import HexColorRange
from aclib.winlib import winapi

from ._functool import findfunc
from ._target import Target
from .window import Window


class CvWindow(Window):

    def _init_(self, hwnd: int):
        super()._init_(hwnd)
        self.__cvready = True
        self.__cvfontlib: FontLib = None
        self.__cvdotsetlib: DotsetLib = None

    @property
    def _cvready(self) -> bool:
        return self.__cvready

    def cvset(self, fontlib: str|None='', dotsetlib: str|None='', flibscale=1.0, dlibscale=1.0) -> Self:
        if fontlib:
            self.__cvfontlib = FontLib.fromfile(fontlib).scale(flibscale)
        if dotsetlib:
            self.__cvdotsetlib = DotsetLib.fromfile(dotsetlib).scale(dlibscale)
        if fontlib is None:
            self.__cvfontlib = None
        if dotsetlib is None:
            self.__cvdotsetlib = None
        return self

    def cvcapture(self, area: _Area=None, savepath='') -> Image | None:
        size, buffer = winapi.CaptureWindow(self.handle, area)
        screenshot = Image.frombuffer(buffer, size)
        if savepath:
            screenshot.tofile(savepath)
        return screenshot

    @findfunc
    def cvfindcolor(self, rgbranges: HexColorRange, areas: _Area|_Areas=None, lpoutput: list=None) -> Target:
        for area in areas:
            screenshot = self.cvcapture(area)
            if not screenshot: continue
            target = screenshot.findcolor(rgbranges)
            if not target: continue
            return target.offset(*area[:2])
        return Target.none

    @findfunc
    def cvfindcolors(self, rgbranges: HexColorRange, areas: _Area|_Areas=None, lpoutput: list=None) -> list[Target]:
        found = []
        for area in areas:
            screenshot = self.cvcapture(area)
            if not screenshot: continue
            targets = screenshot.findcolors(rgbranges)
            found.extend(target.offset(*area[:2]) for target in targets)
        return found

    @findfunc
    def cvfinddotset(self,
        dotsets: str|Sequence[str],
        areas: _Area|_Areas=None,
        matchcolor: Literal[0,1]|HexColorRange=None,
        similarity = 0.9,
        scale = 1.0,
        lpoutput: list=None
    ) -> Target:
        if self.__cvdotsetlib is None:
            return Target.none
        for area in areas:
            screenshot = self.cvcapture(area)
            if not screenshot: continue
            target = screenshot.finddotset(
                self.__cvdotsetlib, dotsets, matchcolor, similarity, scale)
            if target: return target.offset(*area[:2])
        return Target.none

    @findfunc
    def cvfinddotsets(self,
        dotsets: str|Sequence[str],
        areas: _Area|_Areas=None,
        matchcolor: Literal[0,1]|HexColorRange=None,
        similarity = 0.9,
        scale = 1.0,
        ignore_overlaps = False,
        lpoutput: list=None
    ) -> list[Target]:
        found = []
        if self.__cvdotsetlib is None:
            return found
        for area in areas:
            screenshot = self.cvcapture(area)
            if screenshot is None: continue
            targets = screenshot.finddotsets(
                self.__cvdotsetlib, dotsets, matchcolor, similarity, scale, ignore_overlaps)
            found.extend(target.offset(*area[:2]) for target in targets)
        return found

    @findfunc
    def cvfindtext(self,
        texts: str|Sequence[str],
        areas: _Area|_Areas=None,
        matchcolor: Literal[0,1]|HexColorRange=None,
        similarity = 0.9,
        txtdir: Literal[0,1]=0,
        txtwrap = True,
        charscale = 1.0,
        charset: str|Literal['']=None,
        lpoutput: list=None
    ) -> Target:
        if self.__cvfontlib is None:
            return Target.none
        for area in areas:
            screenshot = self.cvcapture(area)
            if screenshot is None: continue
            target = screenshot.findtext(
                self.__cvfontlib, texts, matchcolor, similarity, txtdir, txtwrap, charscale, charset)
            if target: return target.offset(*area[:2])
        return Target.none

    @findfunc
    def cvfindtexts(self,
        texts: str|Sequence[str],
        areas: _Area|_Areas=None,
        matchcolor: Literal[0,1]|HexColorRange=None,
        similarity = 0.9,
        txtdir: Literal[0,1]=0,
        txtwrap = True,
        charscale = 1.0,
        charset: str|Literal['']=None,
        ignore_overlaps = False,
        lpoutput: list=None
    ) -> list[Target]:
        found = []
        if self.__cvfontlib is None:
            return found
        for area in areas:
            screenshot = self.cvcapture(area)
            if screenshot is None: continue
            targets = screenshot.findtexts(
                self.__cvfontlib, texts, matchcolor, similarity, txtdir, txtwrap, charscale, charset, ignore_overlaps)
            found.extend(target.offset(*area[:2]) for target in targets)
        return found

    @findfunc
    def cvocr(self,
        areas: _Area|_Areas=None,
        matchcolor: Literal[0,1]|HexColorRange=None,
        similarity = 0.9,
        txtdir: Literal[0,1]=0,
        txtwrap = True,
        charscale = 1.0,
        charset: str|Literal['']=None,
        lpoutput: list=None
    ) -> list[str]:
        texts = []
        if self.__cvfontlib is None:
            return texts
        for area in areas:
            screenshot = self.cvcapture(area)
            if screenshot is None: continue
            ocrgroups = screenshot.ocr(
                self.__cvfontlib, matchcolor, similarity, txtdir, txtwrap, charscale, charset)
            texts.extend(ocrgroup.join().name for ocrgroup in ocrgroups)
        return texts



from __future__ import annotations
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing import Self, Literal
    from ._typing import _Area, _Areas

import os, itertools
from aclib.dm import DM, DmDotsetLib

from ._functool import findfunc
from ._target import Target
from .window import Window


class DmWindow(Window):

    def _init_(self, hwnd: int):
        super()._init_(hwnd)
        self.__dmready = False
        self.__dmwordlock = False
        self.__dmdotsetlib: DmDotsetLib = None

    @property
    def __dm(self) -> DM:
        del DmWindow.__dm
        self.__dm = DM()
        self.__dm.SetShowErrorMsg(0)
        return self.__dm

    @property
    def _dmready(self) -> bool:
        return self.__dmready

    def dmset(self,
        displaymode: Literal['dx', 'dx2', 'dx3', 'gdi', 'gdi2'] | None = '',
        fontlib: str | None = '',
        dotsetlib: str | None = '',
        dlibscale: float = 1.0
    ) -> Self:
        """传入空字符串时对应参数不做更改，传入None时清除已设置的值"""
        if displaymode:
            self.__dm.UnBindWindow()
            self.__dm.BindWindow(self.handle, displaymode, 'windows', 'windows', 0)
            self.__dmready = True
        if fontlib:
            if not os.path.isfile(fontlib):
                raise FileNotFoundError(
                    f'No such file or directory: {fontlib}')
            self.__dm.SetDict(0, fontlib)
        if dotsetlib:
            self.__dmdotsetlib = DmDotsetLib.fromfile(dotsetlib).scale(dlibscale)
        if displaymode is None:
            self.__dm.UnBindWindow()
            self.__dmready = False
        if fontlib is None:
            self.__dm.ClearDict()
            self.__dmwordlock = False
        if dotsetlib is None:
            self.__dmdotsetlib = None
        return self

    def dmcapture(self, area: _Area=None, savepath: str=''):
        assert self.__dmready, 'dm not set'
        area = area or (0, 0, *self.clientsize)
        self.__dm.Capture(*area, savepath)

    @findfunc
    def dmfindcolor(self, color: str, areas: _Area|_Areas=None, lpoutput: list=None) -> Target:
        for area, color in itertools.product(areas, color.split('|')):
            x, y, success = self.__dm.FindColor(*area, color, 1, 0)
            if success:
                return Target(color, (x, y), (x, y), 1.0)
        return Target.none

    @findfunc
    def dmfindcolors(self, color: str, areas: _Area|_Areas=None, lpoutput: list=None) -> list[Target]:
        found = []
        for area, color in itertools.product(areas, color.split('|')):
            res = self.__dm.FindColorEx(*area, color, 1, 0)
            if not res: continue
            xs, ys = [[int(coor) for coor in coors.split(',')] for coors in res.split('|')]
            for i in range(min(len(xs), len(ys))):
                pos = xs[i], ys[i]
                found.append(Target(color, pos, pos, 1.0))
        return found

    @findfunc
    def dmfindcolorblock(self, colorBlock: str, areas: _Area|_Areas=None, similarity=1, scale=1, lpoutput: list=None) -> Target:
        """ colorBlock: '{w}x{h}x{color}|{w}x{h}x{color}' like '3x5xefefef-101010' """
        for area, colorBlock in itertools.product(areas, colorBlock.split('|')):
            w,h,color = [(round(int(w)*scale), round(int(h)*scale), color) for w,h,color in [(info for info in colorBlock.split('x'))]][0]
            dmColorBlock = ','.join([f'{i%w}|{i//w}|{color}' for i in range(w*h)])
            x, y, success = self.__dm.FindMultiColor(*area, color, dmColorBlock, similarity, 0)
            if success:
                colorBlock = f'{w}x{h}x{color}'
                start = x, y
                end = x+w, y+h
                return Target(colorBlock, start, end, similarity)
        return Target.none

    @findfunc
    def dmfindcolorblocks(self, colorBlock: str, areas: _Area|_Areas=None, similarity=1, scale=1, lpoutput: list=None) -> list[Target]:
        """ colorBlock: '{w}x{h}x{color}|{w}x{h}x{color}' like '3x5xefefef-101010' """
        found = []
        for area, colorBlock in itertools.product(areas, colorBlock.split('|')):
            w,h,color = [(round(int(w)*scale), round(int(h)*scale), color) for w,h,color in [(info for info in colorBlock.split('x'))]][0]
            dmColorBlock = ','.join([f'{i%w}|{i//w}|{color}' for i in range(w*h)])
            res = self.__dm.FindMultiColorEx(*area, color, dmColorBlock, similarity, 0)
            if not res: continue
            xs, ys = [[int(coor) for coor in coors.split(',')] for coors in res.split('|')]
            for i in range(min(len(xs), len(ys))):
                x, y = xs[i], ys[i]
                start = x, y
                end = x+w, y+h
                found.append(Target(colorBlock, start, end, similarity))
        return found

    @findfunc
    def dmfinddotset(self, dotsetname: str, areas: _Area|_Areas=None, color: str=None, similarity=1, scale=1, lpoutput: list=None) -> Target:
        if not self.__dmdotsetlib:
            return Target.none
        for area, dotsetname in itertools.product(areas, dotsetname.split('|')):
            for dotset in self.__dmdotsetlib.group(dotsetname):
                dotset = dotset.scale(scale).asmatchcolor(color)
                x, y, success = self.__dm.FindMultiColor(*area, dotset.matchcolor, dotset.tmpl, similarity, 0)
                if success:
                    start = dotset.getrealpos((x, y))
                    end = start[0] + dotset.width, start[1] + dotset.height
                    return Target(dotsetname, start, end, similarity)
        return Target.none

    @findfunc
    def dmfinddotsets(self, dotsetname: str, areas: _Area|_Areas=None, color: str=None, similarity=1, scale=1, lpoutput: list=None) -> list[Target]:
        found = []
        if not self.__dmdotsetlib:
            return found
        for area, dotsetname in itertools.product(areas, dotsetname.split('|')):
            for dotset in self.__dmdotsetlib.group(dotsetname):
                dotset = dotset.scale(scale).asmatchcolor(color)
                res = self.__dm.FindMultiColorEx(*area, dotset.matchcolor, dotset.tmpl, similarity, 0)
                if not res: continue
                xs, ys = [[int(coor) for coor in coors.split(',')] for coors in res.split('|')]
                for i in range(min(len(xs), len(ys))):
                    x, y = xs[i], ys[i]
                    start = dotset.getrealpos((x, y))
                    end = start[0] + dotset.width, start[1] + dotset.height
                    found.append(Target(dotsetname, start, end, similarity))
        return found

    @findfunc
    def dmfindword(self, texts: str, areas: _Area|_Areas, color: str, similarity=0.9, lpoutput: list=None) -> Target:
        while self.__dmwordlock: pass
        self.__dmwordlock = True
        found = Target.none
        for area in areas:
            x, y, index = self.__dm.FindStrFast(*area, texts, color, similarity)
            if index > -1:
                found = Target(texts.split('|')[index], (x,y), (x,y), similarity)
                break
        self.__dmwordlock = False
        return found

    @findfunc
    def dmfindwords(self, texts: str, areas: _Area|_Areas, color: str, similarity=0.9, lpoutput: list=None) -> list[Target]:
        while self.__dmwordlock: pass
        self.__dmwordlock = True
        found = []
        textList = texts.split('|')
        for area in areas:
            res = self.__dm.FindStrFastEx(*area, texts, color, similarity)
            if not res: continue
            for info in res.split('|'):
                index, x, y = info.split(',')
                word = textList[int(index)]
                pos = int(x), int(y)
                found.append(Target(word, pos, pos, similarity))
        self.__dmwordlock = False
        return found



from .__API__.screen import screen
from .__API__.appwindow import AppWindow
from .__API__.cvwindow import CvWindow
from .__API__.dmwindow import DmWindow
from .__API__._target import Target

class Window(CvWindow, DmWindow): ...



from __future__ import annotations
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing import Self

from typing import overload
from aclib.winlib import winapi

from .basewindow import BaseWindow


__all__ = [
    'Window'
]


class Window(BaseWindow):


    @overload
    def __new__(cls, hwnd: int) -> Self: ...

    @overload
    def __new__(cls, title='', classname='', visible: bool=None) -> Self | None: ...

    def __new__(cls, target: int | str ='', classname='', visivle: bool=None) -> Self | None:
        if isinstance(target, int):
            return super()._new_(target)
        return super()._newwnd_(winapi.FilterWindow(winapi.IterDescendantWindows(0), target, classname, visivle))


    @classmethod
    def findwindows(cls, title='', classname='', visible: bool=None) -> list[Self]:
        return [cls._new_(h) for h in winapi.FilterWindows(winapi.IterDescendantWindows(0), title, classname, visible)]


    @classmethod
    def desktopwindow(cls) -> Self:
        """winapi中定义的DesktopWindow"""
        return cls._new_(winapi.GetDesktopWindow())

    @classmethod
    def desktop(cls) -> Self:
        """显示桌面图标的窗口"""
        return cls._new_(winapi.GetDesktopView())

    @classmethod
    def taskbar(cls) -> Self:
        return cls._new_(winapi.GetTaskbarWindow())


    @classmethod
    def pointwindow(cls, pos: tuple[int, int] = None) -> Self:
        return cls._new_(winapi.GetPointWindow(pos or winapi.GetCursorPos()))

    @classmethod
    def foregroundwindow(cls) -> Self:
        return cls._new_(winapi.GetForegroundWindow())


    def parent(self) -> Self:
        return self._new_(winapi.GetParentWindow(self.handle))

    def root(self) -> Self:
        return self._new_(winapi.GetRootWindow(self.handle))

    def rootowner(self) -> Self:
        return self._new_(winapi.GetRootOwnerWindow(self.handle))


    def prevbrother(self) -> Self | None:
        return self._newwnd_(winapi.GetPrevWindow(self.handle))

    def nextbrother(self) -> Self | None:
        return self._newwnd_(winapi.GetNextWindow(self.handle))

    def brother(self, title='', classname='', visible: bool=None) -> Self | None:
        return self._newwnd_(winapi.FilterWindow(winapi.IterBrotherWindows(self.handle), title, classname, visible))

    def brothers(self, title='', classname='', visible: bool=None) -> list[Self]:
        return [self._new_(h) for h in winapi.FilterWindows(winapi.IterBrotherWindows(self.handle), title, classname, visible)]


    def child(self, title='', classname='', visible: bool=None) -> Self | None:
        return self._newwnd_(winapi.FilterWindow(winapi.IterChildWindows(self.handle), title, classname, visible))

    def children(self, title='', classname='', visible: bool=None) -> list[Self]:
        return [self._new_(h) for h in winapi.FilterWindows(winapi.IterChildWindows(self.handle), title, classname, visible)]


    def descendant(self, title='', classname='', visible: bool=None) -> Self | None:
        return self._newwnd_(winapi.FilterWindow(winapi.IterDescendantWindows(self.handle), title, classname, visible))

    def descendants(self, title='', classname='', visible: bool=None) -> Self | None:
        return [self._new_(h) for h in winapi.FilterWindows(winapi.IterDescendantWindows(self.handle), title, classname, visible)]



from aclib.cv import Dotset, DotsetLib, FontLib
from aclib.autowin.windows import Window

win = Window('一梦江湖', 'Messiah_Game')
print(win)

# d = Dotset.fromimage('1', win.cvcapture((778,277,821,318,)), '208050--2fbf7f', matchcolor=1, cropmode=2, cropmargin=1)
# d.print()

# fl = FontLib.fromfile('tests/num')
# fl.add(d)
# fl.list()
# fl.tofile('tests/num')

win.cvset(dotsetlib='num', fontlib='num')

texts = win.cvocr((528,212,1631,976,), '889999--ffffff', txtwrap=True, similarity=0.7)
print(texts)



from random import randint, random
from time import sleep
from aclib.winlib import winapi


__all__ = [
    'screen'
]


def random_divide(number: float, bynum: int, returnfloat: bool=True):
    nodes = []
    for i in range(bynum-1):
        node = random()*number
        if not returnfloat: node = int(node)
        nodes.append(node)
    nodes.append(number)
    nodes.sort(key=lambda _node:abs(_node))
    results = []
    for i in range(bynum):
        if i==0: results.append(nodes[i])
        else: results.append(nodes[i] - nodes[i-1])
    return results

class Screen(object):
    @property
    def size(self):
        return winapi.GetScreenSize()

    @property
    def mousepos(self):
        return winapi.GetCursorPos()

    def movemouse(self, topos: tuple[int,int]):
        winapi.SetCursorPos(topos)

    def movemouse_likehuman(self, topos: tuple[int,int], minduration: float=0.1, maxduration=0.7):
        startx, starty = self.mousepos
        endx, endy = topos
        xdistance = abs(startx-endx)
        ydistance = abs(starty-endy)
        nmove = randint(int(min(xdistance, ydistance)/2), max(xdistance, ydistance))
        dxlist = random_divide(endx-startx, nmove, returnfloat=False)
        dylist = random_divide(endy-starty, nmove, returnfloat=False)
        dtlist = random_divide(minduration + random() * (maxduration - minduration), nmove, returnfloat=True)
        for dx, dy, dt in zip(dxlist, dylist, dtlist):
            sleep(dt)
            nextpos = startx + dx, starty + dy
            self.movemouse(nextpos)
            startx, starty = nextpos

screen = Screen()



from __future__ import annotations
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing import Self, Callable, Any
    from ._typing import _Pos, _Size

import ctypes
from time import sleep
from functools import wraps
from typing import overload
from aclib.builtins import decorator
from aclib.winlib import winapi, wincon, wintype

from .basewindow import BaseWindow


__all__ = [
    'AppWindow'
]


class AppWindow(BaseWindow):

    class Creator(dict):
        def __init__(self,
            clsname='aclib.winapp', title='', relpos: _Pos=0, size: _Size=0, visible=True, transparency=1.0
        ): super().__init__(locals())

    class Render(dict):
        def __init__(self,
            cursor: int | str = wincon.IDC_ARROW,
            text='', textcplor=0x0, textalign=(3,3), font='', fontsize=18, fillcolor=0xffffff
        ): super().__init__(locals())

    __wndcount = 0
    __wndthread = 0
    __wndevents: dict[int, dict[int, dict[Callable, None]]] = {}  # {hwnd: {msg: {callback: None}}}

    @classmethod
    def __wndproc(cls, hwnd, message, wparam, lparam):
        if message == wincon.WM_CREATE:
            lplistener = ctypes.cast(lparam, ctypes.POINTER(wintype.CREATESTRUCTW)).contents.lpCreateParams
            cls.__wndevents[hwnd] = ctypes.cast(lplistener, ctypes.py_object).value
        if hwnd in cls.__wndevents and message in cls.__wndevents[hwnd]:
            for callback in cls.__wndevents[hwnd][message]:
                callback(hwnd, message, wparam, lparam)
        if message == wincon.WM_NCDESTROY:
            cls.__wndevents.pop(hwnd, None)

    @classmethod
    def __checkthread(cls):
        if not cls.__wndthread:
            cls.__wndthread = winapi.CreateMsgloopThread()

    @classmethod
    def __checkwindow(cls):
        if not cls.__wndcount and cls.__wndthread:
            cls.__wndthread = winapi.DestroyMsgloopThread(cls.__wndthread)

    @classmethod
    def loop(cls):
        while True: sleep(1)

    @classmethod
    def __new(cls, *args, **kwargs):
        window = super()._new_(0)
        window.__init(*args, **kwargs)
        return window

    def __init(self, wndtype: int, parent: Self | int, creator: Creator, render: Render):
        render = render or self.Render()
        creator = creator or self.Creator()
        creator.update({'wndtype': wndtype, 'parent': parent})
        self.__creator = creator
        self.__listener = {}
        self.__trackingmouse = False
        self.cursor = render['cursor']
        self.text = render['text']
        self.textcolor = render['textcplor']
        self.textalign = render['textalign']
        self.font = render['font']
        self.fontsize = render['fontsize']
        self.fillcolor = render['fillcolor']
        for method in filter(lambda k: k.startswith('_AppWindow__on_'), AppWindow.__dict__.keys()):
            self.addmsglistener(getattr(wincon, f'WM_{method[15:].upper()}'), getattr(self, method))

    @decorator.instance_classmethod
    def newoverlapped(parent: Self | int = 0, creator: Creator = None, render: Render = None) -> Self:
        return parent.cls.__new(wincon.WS_OVERLAPPEDWINDOW, parent.self, creator, render)

    @decorator.instance_classmethod
    def newpopup(parent: Self | int, creator: Creator = None, render: Render = None) -> Self:
        return parent.cls.__new(wincon.WS_POPUPWINDOW, parent.self, creator, render)

    @decorator.instance_classmethod
    def newchild(parent: Self | int, creator: Creator = None, render: Render = None) -> Self:
        return parent.cls.__new(wincon.WS_CHILDWINDOW, parent.self, creator, render)

    def __repr__(self):
        wndid = self.handle or hex(id(self))
        wndsummary = self.title or self.classname if self.handle else 'UNCREATED'
        return f'<{self.__class__.__name__}-{wndid} {wndsummary}>'

    def __setattr__(self, key, value):
        super().__setattr__(key, value)
        if self.handle == 0:
            return
        if key in ['fillcolor','text','textcolor','textalign','font','fontsize']:
            winapi.RedrawWindow(self.handle)
        if key == 'cursor':
            winapi.PostMessage(self.handle, wincon.WM_SETCURSOR, self.handle, winapi.MakeLong(wincon.HTCLIENT, wincon.WM_MOUSEMOVE))

    def addmsglistener(self, msg: int | str, callback: Callable[[int, int, int, int], None]):
        if isinstance(msg, str):
            msg = getattr(wincon, f'WM_{msg.upper()}')
        if msg not in self.__listener:
            self.__listener[msg] = {}
        self.__listener[msg][callback] = None

    @overload
    def removemsglistener(self, msg: int | str):
        """remove all listener on msg"""
    @overload
    def removemsglistener(self, msg: int | str, callback: Callable[[int, int, int, int], None]):
        """remove specified listener"""
    def removemsglistener(self, msg: int | str, callback: Callable[[int, int, int, int], None] = None):
        if isinstance(msg, str):
            msg = getattr(wincon, f'WM_{msg.upper()}')
        if msg in self.__listener:
            self.__listener[msg].pop(callback, None)
            if not callback or not self.__listener[msg]:
                self.__listener.pop(msg)

    def create(self) -> Self:
        assert self.handle == 0
        args = self.__creator
        parent = args['parent']
        hparent = parent if isinstance(parent, int) else parent.handle
        dwstyle = args['wndtype'] | wincon.WS_VISIBLE * args['visible']
        dwexstyle = wincon.WS_EX_LAYERED
        if not winapi.GetWindowClassInfo(args['clsname']).hInstance:
            winapi.RegisterWindowClass(args['clsname'], self.__wndproc)
        self.__checkthread()
        winapi.CreateWindowAsync(
            self.__wndthread, hparent, args['clsname'], args['title'],
            args['relpos'], args['size'], dwstyle, dwexstyle, id(self.__listener)
        )   # 如果成功创建窗口，那么在函数返回前，会先执行 __on_create 中的任务
        self.__checkwindow()
        return self

    def __on_create(self, h, m, w, l):
        self._sethandle_(h)
        self.__class__.__wndcount += 1
        self.settransparency(self.__creator['transparency'])
        del self.__creator

    def __on_ncdestroy(self, h, m, w, l):
        self.__class__.__wndcount -= 1
        self.__checkwindow()

    def __on_setcursor(self, h, m, w, l):
        if w == h and winapi.ParseLong(l)[0] == wincon.HTCLIENT:
            winapi.SetCursor(self.cursor)

    def __on_ncpaint(self, *args):
        winapi.Paint(self.handle, self.text, self.textcolor, self.textalign, self.font, self.fontsize, self.fillcolor)

    def __on_mousemove(self, *args):
        if not self.__trackingmouse:
            winapi.TrackMouseEvent(self.handle)
            self.__trackingmouse = True

    def __on_mouseleave(self, *args):
        self.__trackingmouse = False

    @staticmethod
    def __apptask(task):
        @wraps(task)
        def wrappedtask(self, *args, **kwargs):
            if self.handle == 0:
                return None
            if winapi.GetCurrentThreadId() == self.__wndthread:
                return task(self, *args, **kwargs)
            return winapi.RequestMsgloopTask(task, self.__wndthread, self, *args, **kwargs)
        return wrappedtask

    @__apptask
    def capturemouse(self):
        winapi.SetCapture(self.handle)

    @__apptask
    def releasecapture(self):
        winapi.ReleaseCapture()



from __future__ import annotations
from typing import Sequence

__all__ = [
    '_Pos',
    '_Size',
    '_Area',
    '_Areas'
]

_Pos = _Size = tuple[int, int]
_Area = tuple[int, int, int, int]
_Areas = Sequence[_Area]




import inspect, functools

def findfunc(func):

    """
    这是一个装饰器，用来装饰'Window'对象的目标查找方法，以便对其参数进行预处理，
    被装饰的方法名称要以'dm'或者'cv'开头，且带有形参'areas'和'lpoutput'，
    其参数将进行以下预处理：
        1. 断言查找函数的运行环境是否设置
        2. 将传入的单个区域转化为区域列表，并将空的区域默认设置为运行时窗口大小
        3. 自动设置lpoutput的内容，被装饰函数不需要再操作lpoutput
    """

    @functools.wraps(func)
    def wrappedfunc(self, *args, **kwargs):
        uselib = func.__name__[:2]
        ready = getattr(self, f'_{uselib}ready')
        assert ready, f'{uselib} not set'
        arguments = inspect.getcallargs(func, self, *args, **kwargs)
        areas = arguments['areas']
        lpoutput = arguments['lpoutput']
        if not areas or isinstance(areas[0], int):
            areas = [areas]
        arguments['areas'] = [area or (0, 0, *self.clientsize) for area in areas]
        found = func(**arguments)
        if isinstance(lpoutput, list):
            lpoutput.clear()
            lpoutput.extend(found if isinstance(found, list) else [found])
        return found
    return wrappedfunc
